/*
 DialogManager.cc - support cached dialog widgets

 Modified by Will, 2000, 2001: will@opaline.demon.co.uk
 changed 'XmCreateStringSimple' to 'XmCreateStringLtoR' to allow newlines
 WM 'X' directed to actionCallback forcing user to choose button actions.

 Adapted from
 Douglas A. Young, 'Object-oriented Programming with C++ and OSF/Motif'
 2nd ed., Prentice-Hall.  
*/


#include "DialogManager.h"
#include "Application.h"
#include <Xm/MessageB.h>
#include <Xm/Protocols.h>
#include <assert.h>


DialogManager::DialogManager ( const char *name ) : Interface ( name )
{
	// empty
}


Widget DialogManager::getDialog()
{
	Atom WM_DELETE_WINDOW;
	Widget newDialog = NULL;
	
	if ( _w && !XtIsManaged ( _w ) )	// if unused widget exists, return it
		return ( _w );
		
	// get widget from derived class
	newDialog = createDialog ( application->baseWidget() );
	
	if ( newDialog != _w )
	{
		XtAddCallback ( newDialog, XmNokCallback,
						&DialogManager::actionCallback,
						( XtPointer ) this );
						
		XtAddCallback ( newDialog, XmNcancelCallback,
				&DialogManager::actionCallback,
				( XtPointer ) this );
		
		XtAddCallback ( newDialog, XmNhelpCallback,
				&DialogManager::actionCallback,
				( XtPointer ) this );

		// divert WM 'X' action to actionCallback where it's ignored...
		WM_DELETE_WINDOW = XmInternAtom ( XtDisplay ( XtParent ( newDialog ) ), "WM_DELETE_WINDOW", False );
		XmAddWMProtocols ( XtParent ( newDialog ), &WM_DELETE_WINDOW, 1 );
		XmAddWMProtocolCallback ( XtParent ( newDialog ), WM_DELETE_WINDOW, &DialogManager::actionCallback, ( XtPointer ) this );
	}
	
	if ( !_w )		// if first dialog created, save it for re-use
		_w = newDialog;
		
	return ( newDialog );
}


Widget DialogManager::post ( const char *text, void *clientData,
			DialogCallback ok, DialogCallback cancel,
			DialogCallback help )
{
	Widget dialog = getDialog();	// get dialog widget from cache

	assert ( dialog );
	assert ( XtIsSubclass ( dialog, xmMessageBoxWidgetClass ) );

	XmString xmstr = XmStringCreateLtoR ( ( char * ) text, XmFONTLIST_DEFAULT_TAG );
	XtVaSetValues ( dialog, XmNmessageString, xmstr, NULL );

	XmStringFree ( xmstr );
	
	// create object for additional data needed to cache dialog
	DialogCallbackData *dcb = new DialogCallbackData ( clientData, ok,
							cancel, help );
	XtVaSetValues ( dialog, XmNuserData, dcb, NULL );
	
	if ( !help )
	{
		Widget w = XmMessageBoxGetChild ( dialog, XmDIALOG_HELP_BUTTON );
		XtUnmanageChild ( w );
	}

	XtManageChild ( dialog );
	return ( dialog );
}


void DialogManager::actionCallback ( Widget w, XtPointer clientData, XtPointer callData )
{
	XmAnyCallbackStruct *cbs = ( XmAnyCallbackStruct * ) callData;
	DialogManager *obj = ( DialogManager * ) clientData;
	DialogCallback callback;
	DialogCallbackData *dcd;
	
	XtVaGetValues ( w , XmNuserData, &dcd, NULL );

	switch ( cbs->reason )
	{
		case XmCR_OK:
			callback = dcd->ok();
			break;
		case XmCR_CANCEL:
			callback = dcd->cancel();
			break;
		case XmCR_HELP:
			callback = dcd->help();
			break;
		case XmCR_PROTOCOLS:
			return;		// WM protocols trapped here, no action
			break;
	};

	if ( callback != NULL )
		( *callback ) ( dcd->clientData() );
		
	delete dcd;	// delete DialogCallback object
	
	if ( obj->baseWidget() != w )
		XtDestroyWidget ( w );	// if temporary dialogue, delete
}

/*
  Notes:
  
  The post() member function displays a dialog widget; it requires a string to be
  displayed, optional clientData and optional callback functions. The clientData
  is an untyped pointer used to pass data back to the caller on callback
  invocation.

  The post() member function does not install callback functions directly; it
  instantiates a DialogCallbackData object storing pointers to the caller's
  callback functions along with any clientdata. This object is stored in the
  dialog's XmNuserData resource and is retrieved in actionCallback().
  
  actionCallback() is called as a result of user action. 
*/
